package com.ruoyi.agent.service;

import com.alibaba.fastjson2.JSONObject;
import io.milvus.client.MilvusServiceClient;
import io.milvus.grpc.DataType;
import io.milvus.param.ConnectParam;
import io.milvus.param.IndexType;
import io.milvus.param.MetricType;
import io.milvus.param.collection.*;
import io.milvus.param.highlevel.collection.ListCollectionsParam;
import io.milvus.param.index.CreateIndexParam;

import java.text.SimpleDateFormat;
import java.util.List;
import java.util.TimeZone;

public class MilvusService {
    private MilvusServiceClient milvusClient;

    /**
     *  连接milvus知识库
     */
    public MilvusService(String host, int port) {
        this.milvusClient = new MilvusServiceClient(
                ConnectParam.newBuilder()
                        .withHost(host)
                        .withPort(port)
                        .build()
        );
    }

    /**
     * 新建知识库Collection（含Schema、Field、Index）
     */
    public void createCollection(String collectionName, String description, int dimension) {
        FieldType idField = FieldType.newBuilder()
                .withName("id")
                .withDataType(DataType.Int64)
                .withPrimaryKey(true)
                .withAutoID(true)
                .build();

        FieldType fileNameField = FieldType.newBuilder()
                .withName("file_name")
                .withDataType(DataType.VarChar)
                .withMaxLength(256)
                .build();

        FieldType contentField = FieldType.newBuilder()
                .withName("content")
                .withDataType(DataType.VarChar)
                .withMaxLength(65535)
                .build();

        FieldType vectorField = FieldType.newBuilder()
                .withName("embedding")
                .withDataType(DataType.FloatVector)
                .withDimension(dimension)
                .build();

        CreateCollectionParam createCollectionParam = CreateCollectionParam.newBuilder()
                .withCollectionName(collectionName)
                .withDescription(description)
                .addFieldType(idField)
                .addFieldType(fileNameField)
                .addFieldType(contentField)
                .addFieldType(vectorField)
                .build();

        milvusClient.createCollection(createCollectionParam);

        // 创建索引
        CreateIndexParam createIndexParam = CreateIndexParam.newBuilder()
                .withCollectionName(collectionName)
                .withFieldName("embedding")
                .withIndexType(IndexType.IVF_FLAT)
                .withMetricType(MetricType.COSINE)
                .withExtraParam("{\"nlist\": 64}")
                .build();

        milvusClient.createIndex(createIndexParam);
    }

    /**
     * 查询知识库Collection
     */
    public JSONObject getCollectionNames() {
        JSONObject jsonObject = new JSONObject();
        ListCollectionsParam listParam = ListCollectionsParam.newBuilder().build();
        List<String> collectionNames = milvusClient.listCollections(listParam).getData().collectionNames;
        ShowCollectionsParam showParam = ShowCollectionsParam.newBuilder().withCollectionNames(collectionNames).build();
        int size = milvusClient.showCollections(showParam).getData().getCollectionIdsCount();
        for (int i = 0; i < size; i++) {
            jsonObject.put("collectionId", milvusClient.showCollections(showParam).getData().getCollectionIds(i));
            jsonObject.put("collectionName", milvusClient.showCollections(showParam).getData().getCollectionNames(i));
            SimpleDateFormat beijingFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            beijingFormat.setTimeZone(TimeZone.getTimeZone("Asia/Shanghai"));
            String beijingTime = beijingFormat.format(milvusClient.showCollections(showParam).getData().getCreatedUtcTimestamps(i));
            jsonObject.put("createdTime", beijingTime);
        }
        return jsonObject;
    }

    /**
     * 修改知识库Collection
     */
    public void collectionRename(String collectionName, String newCollectionName) {
        RenameCollectionParam renameCollectionReq = RenameCollectionParam.newBuilder()
                .withOldCollectionName(collectionName)
                .withNewCollectionName(newCollectionName)
                .build();

        milvusClient.renameCollection(renameCollectionReq);
    }

    /**
     * 删除知识库Collection
     */
    public void deleteCollectionName(String collectionName) {
        DropCollectionParam param = DropCollectionParam.newBuilder()
                .withCollectionName(collectionName).build();
        milvusClient.dropCollection(param);
    }

    /**
     * 释放知识库Collection
     */
    public void releaseCollectionName(String collectionName) {
        ReleaseCollectionParam param = ReleaseCollectionParam.newBuilder()
                .withCollectionName(collectionName)
                .build();
        milvusClient.releaseCollection(param);
    }

    public void close() {
        milvusClient.close();
    }
}